@@ -59,7 +59,7 @@ char *sys_signame[NSIG] = {
 #endif
 
 int main (int argc, char *argv[]);
-char *basename (char *pathname);
+char *mybasename(char *pathname);
 int signame_to_signum (char *sig);
 int arg_to_signum (char *arg);
 void nosig (char *name);
@@ -80,7 +80,7 @@ int main (int argc, char *argv[])
     int do_pid, do_kill, check_all;
     int *pids, *ip;
 
-    whoami = basename (*argv);
+    whoami = mybasename (*argv);
     numsig = SIGTERM;
     do_pid = (! strcmp (whoami, "pid"));
     do_kill = 0;
@@ -186,7 +186,7 @@ int main (int argc, char *argv[])
     return (errors);
 }
 
-char *basename (char *path)
+char *mybasename (char *path)
 {
     char *cp;
 
