@@ -459,7 +459,7 @@ main(argc, argv)
 				i = read(fd, line, MAXLINE);
 				if (i > 0) {
 					line[i] = '\0';
-					printline(LocalHostName, line);
+					printmulti(LocalHostName, line, i);
 				}
 				else if (i == 0) {
 					dprintf("stream closed (%d)\n", fd);
@@ -481,7 +481,7 @@ main(argc, argv)
 			    (struct sockaddr *) &fromunix, &len);
 			if (i > 0) {
 				line[i] = '\0';
-				printline(LocalHostName, line);
+				printmulti(LocalHostName, line,i);
 			} else if (i < 0 && errno != EINTR)
 				logerror("recvfrom unix");
 		}
@@ -497,7 +497,7 @@ main(argc, argv)
 				extern char *cvthname();
 
 				line[i] = '\0';
-				printline(cvthname(&frominet), line);
+				printmulti(cvthname(&frominet), line,i);
 			} else if (i < 0 && errno != EINTR)
 				logerror("recvfrom inet");
 		}
@@ -514,6 +514,27 @@ usage()
 	exit(1);
 }
 
+/*
+ * Break up null terminated lines for printline.
+ */
+
+printmulti(hname, msg, len)
+        char *hname;
+        char *msg;
+	int  len;
+{
+   int  i;
+   char *pt;
+
+   dprintf("strlen = %d, len = %d\n", strlen(msg), len );
+   for (pt = msg, i = 0; i < len; i++) {
+      if (msg[i] == '\0') {
+	 printline(hname,pt);
+	 pt = &msg[i+1];
+      }
+   }
+}
+
 /*
  * Take a raw input line, decode the message, and print the message
  * on the appropriate log files.
